unit GreyScreen;

{$mode delphi}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  StdCtrls;

type

  { TGreyForm }

  TGreyForm = class(TForm)
    FadeTimer: TTimer;
    procedure FadeTimerTimer(Sender: TObject);
    procedure FormClick(Sender: TObject);
  private
    gfFade                : integer;
    gfFaded               : boolean;
    { private declarations }
  public
    { public declarations }
    property Faded        :boolean   read gfFaded         write gfFaded;
    property fade         :integer   read gfFade          write gfFade;
  end;

var
  GreyForm   : TGreyForm;
  fade       : integer;
  PrevForm   : TForm;
  TopForm    : TForm;

procedure StayOnTop(frm: TForm; Ontop: boolean);
procedure ClearDesktop;
procedure GreyDesktop(frm: TForm; fader: boolean; CoverTaskbar: boolean);


implementation

{$R *.lfm}

{ TGreyForm }

{Grey form fader timer handler.}
procedure TGreyForm.FadeTimerTimer(Sender: TObject);
begin
     FadeTimer.Enabled := false;
     GreyForm.fade := GreyForm.fade + 10;
     GreyForm.AlphaBlendValue := fade;
     if GreyForm.AlphaBlendValue < 180 then begin
        FadeTimer.Enabled := true;
        GreyForm.Faded := true;
     end;
end;

procedure TGreyForm.FormClick(Sender: TObject);
begin
     beep;
end;

{Set active form to stay on top.}
procedure StayOnTop(frm: TForm; Ontop: boolean);
begin
     if Ontop then begin
        frm.FormStyle := fsStayOnTop;
        frm.SetFocus;
     end
     else
        frm.FormStyle := fsNormal;
end;

{Remove grey screen from desktop.}
procedure ClearDesktop;
begin
     if assigned(GreyForm) then begin
        GreyForm.Close;
        FreeAndNil(GreyForm);
        StayOnTop(TopForm, false);
     end;
end;

{Grey desktop and bring form to front.}
procedure GreyDesktop(frm: TForm; fader: boolean; CoverTaskbar: boolean);
begin
     try
        {Create grey form.}
        if not assigned(GreyForm) then begin
           GreyForm := TGreyForm.Create(nil);
           GreyForm.show;
           GreyForm.Faded := false;
        end
        else
           Exit;
        {Set grey form size to screen size.}
        with GreyForm do begin
             {Position form to top left of screen and set to desktop width.}
             top      := Screen.WorkAreaTop;
             left     := Screen.WorkAreaLeft;
             width    := Screen.DesktopWidth;
             {Set height of form to full screen or work area height.}
             if CoverTaskbar then
                height := Screen.DesktopHeight
             else
                height := Screen.WorkAreaHeight;

             TopForm := frm;
             StayOnTop(frm, true);

             if fader then begin
                GreyForm.fade := 10;
                AlphaBlendValue := fade;
                FadeTimer.Enabled := true;
             end else begin
                GreyForm.fade := 10;
                AlphaBlendValue := 180;
                FadeTimer.Enabled := false;
             end;
        end;

     except
           {Free form it exception raised.}
           FreeAndNil(GreyForm);
     end;
end;

end.

