unit AppRoutines;

//{$mode objfpc}{$H+}
{$MODE DELPHI}

interface

uses
  Classes, SysUtils, Windows, ShlObj, lcltype, Forms, comctrls, wininet, UxTheme,
  registry, ActiveX, ComObj, win32proc, win32int, Types, BlowFish, DCPsha256,
  DCPrijndael, odbcconn, mssqlconn, sqldb, db;


type
    MEMORYSTATUSEX = record
      dwLength : DWORD;
      dwMemoryLoad : DWORD;
      ullTotalPhys : uint64;
      ullAvailPhys : uint64;
      ullTotalPageFile : uint64;
      ullAvailPageFile : uint64;
      ullTotalVirtual : uint64;
      ullAvailVirtual : uint64;
      ullAvailExtendedVirtual : uint64;
   end;


{Global functions}
   function GetDesktopFolder : string;
   function GetCurrentUser: String;
   function ShutdownBlockReasonCreate(Handle: hWnd; Msg: lpcwstr): Bool; StdCall; External 'user32';
   function ShutdownBlockReasonDestroy(Handle: hWnd): Bool; StdCall; External 'user32';
   function GetAppVersionStr(filename: string): string;
   function ComponentToString(Component: TComponent): string;
   function StringToComponent(Source: String; Comp: TComponent):TComponent;
   function IsInternetConnected: Boolean;
   function GlobalMemoryStatusEx(var Buffer: MEMORYSTATUSEX): BOOL; stdcall; external 'kernel32' name 'GlobalMemoryStatusEx';
   function GetSystemMem: string;
   function GetLogicalCpuCount: integer;
   function EncryptString(inp: string; key: string): string;
   function DecryptString(inp: string; key: string): string;
   {Registry functions}
   function ReadRegistryString(Root: dword; key: string; field: string): string;
   function ReadRegistryBool(Root: dword; key: string; field: string): boolean;
   function WriteRegistryString(root: dword; key: string; field: string; value: string): boolean;
   function WriteRegistryBool(root: dword; key: string; field: string; value: boolean): boolean;
   function DeleteRegistryField(root: dword; key: string; field: string): boolean;
   function CreateRegistryKey(Root: dword; key: string): boolean;
   function VerifyDBConnection(dbname: string; dbhost: string; dbuser: string; dbpass: string; dbtype: string): boolean;
   function UpperCaseFirstChar(const S: string): string;

{Global procedures}
   procedure SetNodeImage(Node : TTreeNode; index : integer);
   procedure FixPageControl(ACtrl: TObject);
   procedure CreateDesktopShortCut(Target, TargetArguments, ShortcutName, IconPath: string);
   procedure RefreshDesktop;
const
     {$IFDEF UNIX}
     PathDelim = '/';
     {$ELSE}
     PathDelim = '\';
     {$ENDIF}

implementation

uses
    bcconstants;

{Get total system memory.}
function GetSystemMem: string;
var
   MS_Ex: MemoryStatusEx;
begin
     result := '';
     FillChar(MS_Ex, SizeOf(MemoryStatusEx), 0);
     MS_Ex.dwLength := SizeOf(MemoryStatusEx);
     if GlobalMemoryStatusEx(MS_Ex) then
     result := FormatFloat('###.##', MS_Ex.ullTotalPhys / (1024*1024*1024)) + ' GB';
end;


{ Function to retrieve location of host desktop folder.}
function GetDesktopFolder: string;
var
  PIDList: PItemIDList;
  Buffer: array [0..MAX_PATH-1] of Char;
begin
  Result := '';
  SHGetSpecialFolderLocation(0, CSIDL_DESKTOP, PIDList);
  if Assigned(PIDList) then
    if SHGetPathFromIDList(PIDList, Buffer) then
      Result := Buffer;
end;

{Function to retrieve current user name}
function GetCurrentUser: String;
var
  username: String;
  size: DWORD;
begin
  size := 255;
  SetLength(username, size) ;
  if GetUserName(PChar(username), size) then
    Result := Copy(username, 1, size - 1)
  else
    Result := '';
end;

{Procedure change status tree node image}
procedure SetNodeImage(Node : TTreeNode; index : integer);
begin
     node.ImageIndex := index;
     node.SelectedIndex:= index;
end;

{Retreieve application version info}
function GetAppVersionStr(filename: string): string;
var
  my :record
    Dummy: DWord;
    VerInfo: Pointer;
    VerInfoSize: DWord;
    VerValueSize: DWord;
    VerValue:  PVSFixedFileInfo;
    end;
begin
  Result:='';
  my.Dummy:=0; // to keep the compiler happy
  my.VerInfoSize := GetFileVersionInfoSize(PChar(filename), my.Dummy);
  if my.VerInfoSize=0 then
     exit;
  GetMem(my.VerInfo, my.VerInfoSize);
  GetFileVersionInfo(PChar(filename), 0, my.VerInfoSize, my.VerInfo);
  VerQueryValue(my.VerInfo, '\', Pointer(my.VerValue), my.VerValueSize);
  with my.VerValue^ do  begin
     result := IntTostr(dwFileVersionMS shr 16);
     result := result+'.'+   IntTostr(dwFileVersionMS and $FFFF);
     result := result+'.'+   IntTostr(dwFileVersionLS shr 16);
     result := result+'.'+   IntTostr(dwFileVersionLS and $FFFF);
  end;
  FreeMem(my.VerInfo, my.VerInfoSize);
end;

{Save component properties to string}
function ComponentToString(Component: TComponent): string;
var
  BinStream:TMemoryStream;
  StrStream: TStringStream;
  s: string;
begin
  BinStream := TMemoryStream.Create;
  try
    StrStream := TStringStream.Create(s);
    try
      BinStream.WriteComponent(Component);
      BinStream.Seek(0, soFromBeginning);
      ObjectBinaryToText(BinStream, StrStream);
      StrStream.Seek(0, soFromBeginning);
      Result:= StrStream.DataString;
    finally
      StrStream.Free;
    end;
  finally
    BinStream.Free
  end;
end;

{Load component properties for string}
function StringToComponent(Source: String; Comp: TComponent):TComponent;
var
  BinStream:TMemoryStream;
  StrStream: TStringStream;
begin
  StrStream := TStringStream.Create(source);
  try
    BinStream := TMemoryStream.Create;
    try
      Strstream.Seek(0, sofrombeginning);
      ObjectTextToBinary(strStream,binStream);
      BinStream.Seek(0, soFromBeginning);
      Result := BinStream.ReadComponent(Comp);
    finally
      BinStream.Free
    end;
  finally
    StrStream.Free;
  end;
end;

{Find if computer is connected to internet}
function IsInternetConnected: Boolean;
var
  Flags: Windows.DWORD; // flags to pass to API function
begin
  Flags := 0;
  Result := WinInet.InternetGetConnectedState(@Flags, 0);
end;

{Turn themes off for page control}
procedure FixPageControl(ACtrl: TObject);
var
  I: Integer;
begin
  if ACtrl is TPageControl then begin
    UxTheme.SetWindowTheme(TPageControl(ACtrl).Handle, NIL, '');
  end;
end;

{Encrpt string using blowfish crypt, string key required up to 46 characters.}
function EncryptString(inp: string; key: string): string;
var
   DCP_rijndael1: TDCP_rijndael;
begin
     DCP_rijndael1 := TDCP_rijndael.Create(nil);        //we create object.
     DCP_rijndael1.InitStr(key, TDCP_sha256);           //it is initialized
     Result := DCP_rijndael1.EncryptString(inp);        //it is ciphered
     DCP_rijndael1.Burn;                                //we erase info about a key
     DCP_rijndael1.Free;                                //we destroy object
end;

{Decrpt string using blowfish crypt, string key required up to 46 characters.}
function DecryptString(inp: string; key: string): string;
var
   DCP_rijndael1: TDCP_rijndael;
begin
     DCP_rijndael1 := TDCP_rijndael.Create(nil);        //we create object
     DCP_rijndael1.InitStr(key,TDCP_sha256);        //it is initialized
     Result := DCP_rijndael1.DecryptString(inp);      //we decipher
     DCP_rijndael1.Burn;                                 //we erase info about a key
     DCP_rijndael1.Free;                                 //we destroy object
end;

{Read string from resistry}
function ReadRegistryString(Root: dword; key: string; field: string): string;
var
   Registry   : TRegistry;
begin
     result := '';
     Registry := TRegistry.Create(KEY_WRITE OR KEY_WOW64_64KEY);
     try
        Registry.RootKey := root;
        if Registry.OpenKeyReadOnly(key) then
           result := Registry.ReadString(field);
     finally
            Registry.Free;
     end;
end;

{Read string from resistry}
function ReadRegistryBool(Root: dword; key: string; field: string): boolean;
var
   Registry   : TRegistry;
begin
     result := false;
     Registry := TRegistry.Create(KEY_WRITE OR KEY_WOW64_64KEY);
     try
        Registry.RootKey := root;
        if Registry.OpenKeyReadOnly(key) then
           result := Registry.ReadBool(field);
     finally
            Registry.Free;
     end;
end;

{Write string to resistry}
function WriteRegistryString(root: dword; key: string; field: string; value: string): boolean;
var
   Registry   : TRegistry;
begin
     result := false;
     Registry := TRegistry.Create(KEY_WRITE OR KEY_WOW64_64KEY);
     try
        Registry.RootKey:= root;
        if Registry.OpenKey(key, true) then
        begin
             Registry.WriteString(field, value);
             result := true;
        end;
     finally
            Registry.Free;
     end;
end;

{Write boolean to resistry}
function WriteRegistryBool(root: dword; key: string; field: string; value: boolean): boolean;
var
   Registry   : TRegistry;
begin
     result := false;
     Registry := TRegistry.Create(KEY_WRITE OR KEY_WOW64_64KEY);
     try
        Registry.RootKey:= root;
        if Registry.OpenKey(key, true) then
        begin
             Registry.WriteBool(field, value);
             result := true;
        end;
     finally
            Registry.Free;
     end;
end;

{Delete registry string value.}
function DeleteRegistryField(root: dword; key: string; field: string): boolean;
var
   Registry   : TRegistry;
begin
     result := false;
     Registry := TRegistry.Create(KEY_WRITE OR KEY_WOW64_64KEY);
     try
        Registry.RootKey := root;
        if Registry.OpenKey(key, false) then
        begin
             Registry.DeleteValue(field);
             result := true;
        end;
     finally
            Registry.Free;
     end;
end;

{Create resistry key}
function CreateRegistryKey(Root: dword; key: string): boolean;
var
   Registry   : TRegistry;
begin
     result := false;
     Registry := TRegistry.Create(KEY_WRITE OR KEY_WOW64_64KEY);
     try
        Registry.RootKey:= root;
        if Registry.OpenKey(key, true) then
           result := true;
     finally
            Registry.Free;
     end;
end;

{Create desktop link}
procedure CreateDesktopShortCut(Target, TargetArguments, ShortcutName, IconPath : string);
var
  IObject: IUnknown;
  ISLink: IShellLink;
  IPFile: IPersistFile;
  PIDL: PItemIDList;
  InFolder: array[0..MAX_PATH] of Char;
  LinkName: WideString;
begin
  { Creates an instance of IShellLink }
  IObject := CreateComObject(CLSID_ShellLink);
  ISLink := IObject as IShellLink;
  IPFile := IObject as IPersistFile;

  ISLink.SetPath(pChar(Target));
  ISLink.SetArguments(pChar(TargetArguments));
  ISLink.SetWorkingDirectory(pChar(ExtractFilePath(Target)));
  ISLink.SetIconLocation(pChar(IconPath), 0);

  { Get the desktop location }
  SHGetSpecialFolderLocation(0, CSIDL_DESKTOPDIRECTORY, PIDL);
  SHGetPathFromIDList(PIDL, InFolder);
  LinkName := InFolder + PathDelim + ShortcutName + '.lnk';

  { Create the link }
  IPFile.Save(PWChar(LinkName), false);
end;


{Refresh windows desktop.}
procedure RefreshDesktop;
begin
  SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, nil, nil);
end;

{Check if user has write access to directory}
function DirectoryWriteable (lDir: string): boolean;
var
    i : Longint;
    lFilename: string;
Begin
     result := false;
     if length(lDir) < 1 then
        exit;
     if lDir[length(lDir)] <>  PathDelim then
        lFilename := lDir + pathdelim + 'dummy.dum'
     else
         lFilename := lDir + 'dummy.dum';
     if fileexists(lFilename) then
        exit; //do not overwrite existing file
     i := FileCreate(lFilename);
     if i = -1 then
     Halt(1);
     FileClose(i);
     DeleteFile(PChar(lFilename));
     result := true;
end;

{Verify database connection.}
function VerifyDBConnection(dbname: string; dbhost: string; dbuser: string;
  dbpass: string; dbtype: string): boolean;
var
   ODBCConnection : TODBCConnection;
begin
     {Create and configure database connection object.}
     ODBCConnection := TODBCConnection.Create(nil);
     try
        ODBCConnection.Driver := dbtype;                    // Set connection type.
        ODBCConnection.Params.Add('Server=' + dbhost);      // Set host name.
        ODBCConnection.Params.Add('DBQ=' + dbname);         // Set databasename.
        ODBCConnection.Params.Add('ExtendedAnsiSQL=1');
        ODBCConnection.Params.Add('CHARSET=ansi');
        ODBCConnection.UserName       := dbuser;            // Set user name.
        ODBCConnection.Password       := dbpass;            // Set password.
        {Attempt connection.}
        try
           ODBCConnection.Connected := True;
           ODBCConnection.KeepConnection := true;
           {Check if connection established.}
           result := ODBCConnection.Connected;
        except
              {ignore exception.}
        end;
     finally
            ODBCConnection.Connected := false;
            ODBCConnection.Free;
     end;
end;

{Convert first letter of a string to capitals.}
function UpperCaseFirstChar(const S: string): string;
begin
     {First convert string to lower case characters.}
     result := LowerCase(s);
     {Fianlly convert first character to upper case and return result.}
     if Length(Result) > 0 then
        Result[1] := UpCase(Result[1]);
end;

{ Function to return logical processor core count.}
function GetLogicalCpuCount: integer;
// returns a good default for the number of threads on this system
{$IF defined(windows)}
//returns total number of processors available to system including logical hyperthreaded processors
var
  i: Integer;
  ProcessAffinityMask, SystemAffinityMask: DWORD_PTR;
  Mask: DWORD;
  SystemInfo: SYSTEM_INFO;
begin
  if GetProcessAffinityMask(GetCurrentProcess, ProcessAffinityMask, SystemAffinityMask)
  then begin
    Result := 0;
    for i := 0 to 31 do begin
      Mask := DWord(1) shl i;
      if (ProcessAffinityMask and Mask)<>0 then
        inc(Result);
    end;
  end else begin
    //can't get the affinity mask so we just report the total number of processors
    GetSystemInfo(SystemInfo);
    Result := SystemInfo.dwNumberOfProcessors;
  end;
end;
{$ELSEIF defined(UNTESTEDsolaris)}
  begin
    t = sysconf(_SC_NPROC_ONLN);
  end;
{$ELSEIF defined(freebsd) or defined(darwin)}
var
  mib: array[0..1] of cint;
  len: cint;
  t: cint;
begin
  mib[0] := CTL_HW;
  mib[1] := HW_NCPU;
  len := sizeof(t);
  fpsysctl(pchar(@mib), 2, @t, @len, Nil, 0);
  Result:=t;
end;
{$ELSEIF defined(linux)}
  begin
    Result:=sysconf(_SC_NPROCESSORS_ONLN);
  end;

{$ELSE}
  begin
    Result:=1;
  end;
{$ENDIF}



end.

