unit bcoptions;

//{$mode objfpc}{$H+}
{$MODE DELPHI}{$H+}

interface

uses
  Classes, SysUtils, lcltype, forms, win32int, InterfaceBase, registry,
  RunElevatedSupport, StringVar, odbcconn, mssqlconn, sqldb, db, StdCtrls,
  windows, ipservices;

type
    { TToolRecord object }
    TToolRecord = class(TComponent)
      private
        trCal                          : boolean;
        trCalDate                      : TDateTime;
        trDesc                         : string;
        trExpDate                      : TDateTime;
        trID                           : string;
        trPrevCal                      : TDateTime;
        trPrevExp                      : TDateTime;
        trSet                          : string;
        trType                         : string;
        trUpdate                       : boolean;

      public
        property ToolID                : string       read trID           write trID;
        property ToolType              : string       read trType         write trType;
        property ToolDescription       : string       read trDesc         write trDesc;
        property ToolSetting           : string       read trSet          write trSet;
        property ToolCalRequired       : boolean      read trCal          write trCal;
        property ToolCalDate           : TDateTime    read trCalDate      write trCalDate;
        property ToolExpDate           : TDateTime    read trExpDate      write trExpDate;
        property NewTool               : boolean      read trUpdate       write trUpdate;
        property PrevCalDate           : TDateTime    read trPrevCal      write trPrevCal;
        property PrevExpDate           : TDateTime    read trPrevExp      write trPrevExp;

        procedure GetToolTypeList(list: TStrings);
        procedure GetToolDescriptionList(List: tStrings);
        procedure GetToolSetting(List: tStrings);
      
        constructor Create(aOwner: TComponent); override;
        destructor Destroy; override;
    end;

    { TWorkbench }
    TWorkbench = class(TComponent)
      private
        bBchIcon                       : integer;
        bBench                         : boolean;
        bCons                          : boolean;
        bEFOD                          : boolean;
        bEN                            : integer;
        bEOFF                          : boolean;
        bETime                         : boolean;
        bEWIP                          : boolean;
        bEXT                           : boolean;
        bfix                           : integer;
        bFN                            : string;
        bJacket                        : boolean;
        blaunch                        : integer;
        bLoc                           : string;
        bManIcon                       : integer;
        bmax                           : integer;
        bmin                           : integer;
        bName                          : string;
        bPrompt                        : integer;
        bPW                            : string;
        bRA                            : boolean;
        brand                          : integer;
        brequired                      : integer;
        bSFOD                          : boolean;
        bSN                            : string;
        bStart                         : boolean;
        bStraps                        : boolean;
        bTlsIcon                       : integer;
        bTlsStr                        : integer;
        bTools                         : boolean;
        bUN                            : string;

      public
        property RandomInterval        : integer      read brand        write brand;

        {Current user properties.}
        property Username              : string       read bUN        write bUN;
        property EmployeeNumber        : integer      read bEN        write bEN;
        property Firstname             : string       read bFN        write bFN;
        property Surname               : string       read bSN        write bSN;
        property Password              : string       read bPW        write bPW;
        property RestrictedAccess      : boolean      read bRA        write bRA;
        {Start shift properties.}
        property ShiftStartComplete    : boolean      read bStart     write bStart;
        property ToolsConfirmed        : boolean      read bTools     write bTools;
        property ConsumablesConfirmed  : boolean      read bCons      write bCons;
        property ESDStrapsConfirmed    : boolean      read bStraps    write bStraps;
        property ESDBenchConfirmed     : boolean      read bBench     write bBench;
        property ESDJacketConfirmed    : boolean      read bJacket    write bJacket;
        property StartFODConfirmed     : boolean      read bSFOD      write bSFOD;
        property ExtractionConfirmed   : boolean      read bEXT       write bEXT;
        {End of shift properties.}
        property EOSEquipmentOff       : boolean      read bEOFF      write bEOFF;
        property EOSFODConfirmed       : boolean      read bEFOD      write bEFOD;
        property EOSWIP                : boolean      read bEWIP      write bEWIP;
        property EOSTimecard           : boolean      read bETime     write bETime;

        constructor Create(aOwner: TComponent); override;
        destructor Destroy; override;

        procedure ReadRegistrySettings;
        procedure SaveRegistrySettings;
        procedure SetDefaultValues;
      published
        {Workbench properties.}
        property BenchName             : string       read bName        write bName;
        property BenchLocation         : string       read bLoc         write bLoc;
        {Benchcheck properties.}
        property BenchcheckIndex       : integer      read brequired    write brequired;
        property BenchcheckLaunchIndex : integer      read blaunch      write blaunch;
        property FixedInterval         : integer      read bfix         write bfix;
        property MinInterval           : integer      read bmin         Write bmin;
        property MaxInterval           : integer      read bmax         write bmax;
        property BenchIconIndex        : integer      read bBchIcon     write bBchIcon;
        property CalibrationPrompt     : integer      read bPrompt      write bPrompt;
        {Tool register properties.}
        property ToolsIconIndex        : integer      read bTlsIcon     write bTlsIcon;
        property RunToolAtStart        : integer      read bTlsStr      write bTlsStr;
        {Manager properties.}
        property ManagerIconIndex      : integer      read bManIcon     write bManIcon;
    end;

    { TSettings }
    TSettings = class(TComponent)
      private
        opAdmin                        : boolean;
        opAutoStart                    : integer;
        opBchDB                        : string;
        opBchDBVer                     : boolean;
        opBchLoc                       : stringlist;
        opBchPass                      : string;
        opBlkWIn                       : integer;
        opBlock: boolean;
        opBncErr                       : StringList;
        opBchStat                      : boolean;
        opDBDir                        : string;
        opDBDrv                        : string;
        opDBErr                        : stringlist;
        opEnvErr                       : StringList;
        opEnvStat                      : boolean;
        opFade                         : integer;
        opFault                        : boolean;
        opFileCng                      : boolean;
        opGrey                         : integer;
        opAdminAcc                     : boolean;
        opManLau                       : boolean;
        opMFHandle                     : HWND;
        opRegCng                       : boolean;
        opRestrict                     : integer;
        opSQL                          : integer;
        opStartFrom                    : integer;
        opTask                         : integer;
        opTlsDB                        : string;
        opTlsDBPass                    : string;
        opTlsDBVer                     : boolean;
        opUAC                          : boolean;
        opUpdate                       : integer;
        opHostDrv                      : string;
        opDesktop                      : string;
        opAppHandle                    : HWND;
        opSplash                       : integer;
        opHostDir                      : string;
        opAppStat                      : boolean;
        opDBStat                       : boolean;
        opHostConf                     : boolean;
        opCoreDB                       : string;
        opUserPass                     : string;
        opCoreDBVer                    : boolean;
        opWinMaj                       : integer;
        opWinMin                       : integer;
        opWInBuild                     : integer;
        opWinName                      : string;
        opWinEd                        : string;
        opWinCSD                       : string;
        opProc                         : string;
        opScreenH                      : integer;
        opScreenW                      : integer;
        opSourceDir                    : string;
        opSourceDrv                    : string;
        opAppErr                       : StringList;

        procedure SetAppErr(const Value: StringList);
        procedure SetBchLoc(AValue: stringlist);
        procedure SetBncErr(AValue: StringList);
        procedure SetDBErr(AValue: stringlist);
        procedure SetEnvErr(AValue: StringList);

      public
        {Application properties}
        property AppHandle             : HWND       read opAppHandle  write opAppHandle;
        property MainFormHandle        : HWND       read opMFHandle   write opMFhandle;
        property RegistryChange        : boolean    read opRegCng     write opRegCng;
        property FileChange            : boolean    read opFileCng    write opFileCng;
        property BenchLocation         : stringlist read opBchLoc     write SetBchLoc;
        property ConfigFault           : boolean    read opFault      write opFault;

        {Database verification properties.}
        property CoreDBVer             : boolean    read opCoreDBVer  write opCoreDBVer;
        property BenchcheckDBVer       : boolean    read opBchDBVer   write opBchDBVer;
        property ToolsDBVer            : boolean    read opTlsDBVer   write opTlsDBVer;

        {System properties}
        property HostRootDrive         : string     read opHostDrv    write opHostDrv;
        property DesktopDir            : string     read opDesktop    write opDesktop;
        property WindowsVerMajor       : integer    read opWinMaj     write opWinMaj;
        property WindowsVerMinor       : integer    read opWinMin     write opWinMin;
        property WindowsBulid          : integer    read opWInBuild   write opWinBuild;
        property WindowsCSDVersion     : string     read opWinCSD     write opWinCSD;
        property WindowsName           : string     read opWinName    write opWinName;
        property WindowsEdition        : string     read opWinEd      write opWinEd;
        property ProcessorName         : string     read opProc       write opProc;
        property ScreenHeight          : integer    read opScreenH    write opScreenH;
        property ScreenWidth           : integer    read opScreenW    write opScreenW;
        property UACEnabled            : boolean    read opUAC        write opUAC;
        property RunningAdmin          : boolean    read opAdmin      write opAdmin;
        property ManualLaunch          : boolean    read opManLau     write opManLau;
        property IsAdminAcount         : boolean    read opAdminAcc   write opAdminAcc;
        property BlockShutdown         : boolean    read opBlock      write opBlock;

        {Environmental status check flags.}
        property EnvStatus             : boolean    read opEnvStat    write opEnvStat;
        property EnvErrorList          : stringlist read opEnvErr     write SetEnvErr;

        {Workbench statue check flag.}
        property WorkBenchStatus       : boolean    read opBchStat    write opBchStat;
        property WorkbenchErrorList    : stringlist read opBncErr     write SetBncErr;

        {Application status check flags}
        property AppStatus             : boolean    read opAppStat    write opAppStat;
        property AppErrorList          : stringlist read opAppErr     write SetAppErr;
        property HostConfigured        : boolean    read opHostConf   write opHostConf;

        {Database status checking}
        property DBStatus              : boolean    read opDBStat     write opDBStat;
        property DBErrorList           : stringlist read opDBErr      write SetDBErr;

        constructor Create(aOwner: TComponent); override;
        destructor Destroy; override;

        procedure ReadRegistrySettings;
        procedure ReadFileSettings;
        procedure SaveRegistrySettings;
        procedure SaveFileSettings;
        procedure GetSystemSettings;
        procedure SetDefaultValues;

      published
        {Stored environmental properties}
        property HostAppDirectory      : string     read opHostDir    write opHostDir;
        property RemoteDirectory       : string     read opSourceDir  write opSourceDir;
        property RemoteDrive           : string     read opSourceDrv  write opSourceDrv;
        property DatabaseDrive         : string     read opDBDrv      write opDBDrv;
        property DatabaseDir           : string     read opDBDir      write opDBDir;
        {Stored application settings properties.}
        property StartFromIndex        : integer    read opStartFrom  write opStartFrom;
        property AutoUpdateIndex       : integer    read opUpdate     write opUpdate;
        property AutoStartIndex        : integer    read opAutoStart  write opAutoStart;
        property GreyScreenIndex       : integer    read opGrey       write opGrey;
        property GreyFadeIndex         : integer    read opFade       write opFade;
        property GreyTaskbarIndex      : integer    read opTask       write opTask;
        property ShowSplashIndex       : integer    read opSplash     write opSplash;
        property SQLServerIndex        : integer    read opSQL        write opSQL;

        {Stored user database properties.}
        property CoreDB                : string     read opCoreDB     write opCoreDB;
        property CoreDBPassword        : string     read opUserPass   write opUserPass;
        property RestrictAccessDefault : integer    read opRestrict   write opRestrict;
        {Stored bench check database properties.}
        property BenchcheckDB          : string     read opBchDB      write opBchDB;
        property BenchcheckDBPassword  : string     read opBchPass    write opBchPass;
        property BlockWindowsIndex     : integer     read opBlkWIn     write opBlkWin;
        {Stored tool register database properties.}
        property ToolsDB               : string     read opTlsDB      write opTlsDB;
        property ToolsDBPassword       : string     read opTlsDBPass  write opTlsDBPass;
    end;

{Global objects.}
var
   {Application property objects.}
   Settings                       : TSettings;
   WorkBench                      : TWorkBench;
   {Windows message holders.}
   CoreMessage                    : UINT;
   BenchMessage                   : UINT;
   ToolsMessage                   : UINT;
   UserInfoMessage                : UINT;
   ManagerMessage                 : UINT;
   {User dateabase components.}
   UserDatabaseConnection         : TODBCConnection  ;
   UserTransaction                : TSQLTransaction;
   UserQuery                      : TSQLQUery;
   {Workbench database components.}
   WorkbenchDatabaseConnection    : TODBCConnection  ;
   WorkbenchTransaction           : TSQLTransaction;
   WorkbenchQuery                 : TSQLQUery;
   {Tools dateabase components.}
   ToolsDatabaseConnection        : TODBCConnection  ;
   ToolsTransaction               : TSQLTransaction;
   ToolsQuery                     : TSQLQuery;
   ToolRecord                     : TToolRecord;
   {Network interface objects.}
   NetworkInterfaceList           : TNetworkInterfaceList;

{Application specific global routines.}
procedure UpdateGreyScreen(frm: TForm);
procedure CloseUserConnection;
procedure CloseToolsConnection;
procedure CloseWorkbenchConnection;

function  CreateUserDatabaseConnection(parent: TForm): boolean;
function  CreateToolsDatabaseConnection(parent: TForm): boolean;
function  CreateWorkbenchDatabaseConnection(parent: TForm): boolean;

implementation

uses
    approutines, bcconstants, greyscreen, bcmain;

{ TToolRecord }

procedure TToolRecord.GetToolTypeList(list: TStrings);
var
   i : integer;
begin
     {Clear tool type list.}
     list.Clear;
     {Create tools register database object.}
     if CreateToolsDatabaseConnection(nil) then begin
        {Set SQL search parameters and run query.}
        ToolsQuery.Close;
        ToolsQuery.SQL.Text := QGetToolTypeList;
        ToolsQuery.Open;
        {If items found then copt to string list.}
        if not ToolsQuery.EOF then begin
           for i := 0 to ToolsQuery.RecordCount - 1 do begin
               list.Add(ToolsQuery.FieldByName('type').AsString);
               ToolsQuery.Next;
           end;
        end;
     end;
     CloseToolsConnection;
end;

procedure TToolRecord.GetToolDescriptionList(List: tStrings);
var
   i : integer;
begin
     {Clear tool type list.}
     list.Clear;
     {Create tools register database object.}
     if CreateToolsDatabaseConnection(nil) then begin
        {Set SQL search parameters and run query.}
        ToolsQuery.Close;
        ToolsQuery.SQL.Text := QGetToolDescriptionsList;
        ToolsQuery.ParamByName('type').AsString := ToolRecord.ToolType;
        ToolsQuery.Open;
        {If items found then copt to string list.}
        if not ToolsQuery.EOF then begin
           for i := 0 to ToolsQuery.RecordCount - 1 do begin
               list.Add(ToolsQuery.FieldByName('description').AsString);
               ToolsQuery.Next;
           end;
        end;
     end;
     CloseToolsConnection;
end;

procedure TToolRecord.GetToolSetting(List: tStrings);
var
   i : integer;
begin
     {Clear tool type list.}
     list.Clear;
     {Create tools register database object.}
     if CreateToolsDatabaseConnection(nil) then begin
        {Set SQL search parameters and run query.}
        ToolsQuery.Close;
        ToolsQuery.SQL.Text := QGetToolSettingsList;
        ToolsQuery.ParamByName('des').AsString := ToolRecord.ToolDescription;
        ToolsQuery.Open;
        {If items found then copt to string list.}
        if not ToolsQuery.EOF then begin
           for i := 0 to ToolsQuery.RecordCount - 1 do begin
               list.Add(ToolsQuery.FieldByName('setting').AsString);
               ToolsQuery.Next;
           end;
        end;
     end;
     CloseToolsConnection;
end;

constructor TToolRecord.Create(aOwner: TComponent);
begin
     inherited Create(aOwner);
end;

destructor TToolRecord.Destroy;
begin
     inherited Destroy;
end;


{ TWorkbench }

constructor TWorkbench.Create(aOwner: TComponent);
begin
     inherited Create(aOwner);
     {Load default values.}
     SetDefaultValues;
end;

destructor TWorkbench.Destroy;
begin
     inherited Destroy;
end;

procedure TWorkbench.ReadRegistrySettings;
begin
     if ReadRegistryString(RegHKLM, RegWorkBenchSuite, WorkBenchSettings) <> '' then
        StringToComponent(DecryptString(ReadRegistryString(RegHKLM, RegWorkBenchSuite,
                              WorkBenchSettings), CryptKey), WorkBench);
end;

procedure TWorkbench.SaveRegistrySettings;
begin
     WriteRegistryString(RegHKLM, RegWorkBenchSuite, WorkBenchSettings,
               EncryptString(ComponentToString(WorkBench), CryptKey));
end;

procedure TWorkbench.SetDefaultValues;
begin
     {Set default index values.}
     brequired         := 0;
     blaunch           := 0;
     bBchIcon          := 0;
     bTlsIcon          := 0;
     bManIcon          := 1;
     bTlsStr           := 1;
     {Set default time intervals in minutes.}
     bfix              := 2;
     bmin              := 1;
     bmax              := 5;
     {Set default calibration prompt.}
     bPrompt           := 7;
     Username := GetCurrentUser;
end;

{Application options component}
constructor TSettings.Create(aOwner: TComponent);
begin
     inherited Create(aOwner);
     GetSystemSettings;
     SetDefaultValues;
end;

procedure TSettings.GetSystemSettings;
var
   Screen     : TScreen;
begin
     {Retrieve host property values}
     opHostDrv          := IncludeTrailingPathDelimiter(ExtractFileDrive(GetDesktopFolder));
     opDesktop       := IncludeTrailingPathDelimiter(ExtractRelativePath(opHostDrv, GetDesktopFolder));
     {Get operating system info}
     opWinMaj        := Win32MajorVersion;
     opWinMin        := Win32MinorVersion;
     opWinBuild      := Win32BuildNumber;
     opWinCSD        := Win32CSDVersion;
     opUAC           := IsUACEnabled;
     if (opWinMaj = 5) and (opWinMin = 1) then
                opWinName := 'Microsoft Windows XP'
     else if (opWinMaj = 6) and (opWinMin = 0) then
                opWinName := 'Microsoft Windows Vista'
     else if (opWinMaj = 6) and (opWinMin = 1) then
                opWinName := 'Microsoft Windows 7'
     else if (opWinMaj = 6) and (opWinMin = 2) then
                opWinName := 'Microsoft Windows 8'
     else if (opWinMaj = 6) and (opWinMin = 3) then
                opWinName := 'Microsoft Windows 8.1'
     else if opWinMaj = 10 then
                opWinName := 'Microsoft Windows 10';
     opWinEd := ReadRegistryString(RegHKLM, RegCurrentVersion, 'EditionID');
     {Get processor name}
     opProc := ReadRegistryString(RegHKLM, RegCPU, 'ProcessorNameString');
     {Get screen resolution.}
     Screen := TScreen.Create(nil);
     try
        opScreenH := Screen.Height;
        opScreenW := Screen.Width;
     finally
            Screen.free;
     end;
     {Get current user.}
     opAdminAcc      := IsAdministratorAccount;
     {Retrieve application properties}
     opAppHandle     := WidgetSet.AppHandle; // Application handle.
     opMFHandle      := Application.MainFormHandle;
     opSourceDrv     := IncludeTrailingPathDelimiter(ExtractFileDrive(Application.ExeName));
     opSourceDir     := IncludeTrailingPathDelimiter(ExtractRelativePath(opSourceDrv,
                                ExtractFilePath(Application.ExeName)));
     if opSourceDir = '\' then opSourceDir := '';
     opAdmin         := IsAdministrator;
     if not opHostConf then opAppErr.Add('Not configured on this machine.');
     {Load bench location file.}
     if FileExists(opSourceDrv + opSourceDir + WorkbenchLocation) then
        BenchLocation.LoadFromFile(RemoteDrive + RemoteDirectory + WorkbenchLocation);

end;

procedure TSettings.SetDefaultValues;
begin
     {Set default booleand properties}
     opEnvStat       := true;
     opAppStat       := true;
     opBchStat       := true;
     opDBStat        := true;
     opHostConf      := false;
     opRegCng        := false;
     opFileCng       := false;
     opManLau        := false;
     opCoreDBVer     := false;
     opBchDBVer      := false;
     opTlsDBVer      := false;
     opBlock         := false;
     {Set default numerical properties.}
     opStartFrom     := 0;
     opUpdate        := 1;
     opAutoStart     := 1;
     opGrey          := 0;
     opFade          := 0;
     opTask          := 0;
     opSplash        := 0;
     opRestrict      := 0;
     opBlkWin        := 1;
     {Set default string properties}
     opHostDir       := Application.Title + '\';
     opDBDir         := opSourceDir + 'DataBase\';
     opDBDrv         := opSourceDrv;
end;

destructor TSettings.Destroy;
begin
     // Insert object to be freed before component close.
     inherited Destroy;
end;

{Read saved settings object from registry.}
procedure TSettings.ReadRegistrySettings;
begin
     if ReadRegistryString(RegHKLM, RegWorkBenchSuite, ApplicationSettings) <> '' then
        StringToComponent(DecryptString(ReadRegistryString(RegHKLM, RegWorkBenchSuite,
                        ApplicationSettings), CryptKey), Settings);
end;

{Convert settings object to string and save to registry.}
procedure TSettings.SaveRegistrySettings;
begin
     WriteRegistryString(RegHKLM, RegWorkBenchSuite, ApplicationSettings,
             EncryptString(ComponentToString(Settings), CryptKey));
end;

{Read saved setting object from file.}
procedure TSettings.ReadFileSettings;
var
   fstream : TFileStream;
begin
     if FileExists(Settings.RemoteDrive + Settings.RemoteDirectory + WorkbenchConfig) then begin
        fstream := TFileStream.Create(Settings.RemoteDrive + Settings.RemoteDirectory + WorkbenchConfig, fmOpenRead);
        try
           fstream.Position := 0;
           StringToComponent(DecryptString(fstream.ReadAnsiString, CryptKey), settings);
        finally
               fstream.Free;
        end;
     end;
end;
{Convert settings object to string and save to file.}
procedure TSettings.SaveFileSettings;
var
   fstream : TFileStream;
begin
     if FileExists(Settings.RemoteDrive + Settings.RemoteDirectory + WorkbenchConfig) then
        DeleteFile(PChar(Settings.RemoteDrive + Settings.RemoteDirectory + WorkbenchConfig));
     fstream := TFileStream.Create(Settings.RemoteDrive + Settings.RemoteDirectory + WorkbenchConfig, fmCreate);
     try
        fstream.WriteAnsiString(EncryptString(ComponentToString(Settings), CryptKey));
     finally
            fstream.Free;
     end;
end;

procedure TSettings.SetAppErr(const Value: StringList);
begin
     if opAppErr = Value then Exit;
     opAppErr := Value;
end;

procedure TSettings.SetBchLoc(AValue: stringlist);
begin
     if opBchLoc=AValue then Exit;
     opBchLoc:=AValue;
end;

procedure TSettings.SetBncErr(AValue: StringList);
begin
     if opBncErr=AValue then Exit;
     opBncErr:=AValue;
end;

procedure TSettings.SetDBErr(AValue: stringlist);
begin
     if opDBErr = AValue then Exit;
     opDBErr := AValue;
end;

procedure TSettings.SetEnvErr(AValue: StringList);
begin
     if opEnvErr = AValue then Exit;
     opEnvErr := AValue;
end;

{Update desktop grey out for specified form.}
procedure UpdateGreyScreen(frm: TForm);
var
   fade, bar : boolean;
begin
     {Clear grey form.}
     ClearDesktop;
     {Display updated grey form.}
     if Settings.GreyScreenIndex = 0 then begin
        {Set fade value.}
        if Settings.GreyFadeIndex = 0 then fade := true;
        if Settings.GreyFadeIndex = 1 then fade := false;
        {Set task bar access.}
        if Settings.GreyTaskbarIndex = 0 then bar := false;
        if Settings.GreyTaskbarIndex = 1 then bar := true;
        {Display grey form.}
        GreyDesktop(frm, fade, bar);
     end;
end;

{Create user database connection.}
function CreateUserDatabaseConnection(parent: TForm): boolean;
begin
     result := false;
     {Create user database connection component.}
     UserDatabaseConnection := TODBCConnection  .Create(parent);
     try
        if Settings.SQLServerIndex = 0 then begin;
           UserDatabaseConnection.Driver := 'Microsoft Access Driver (*.mdb)';
           UserDatabaseConnection.Params.Add('DBQ=' + Settings.DatabaseDrive +
                                           Settings.DatabaseDir +
                                           Settings.CoreDB);
           UserDatabaseConnection.Params.Add('ExtendedAnsiSQL=1');
           UserDatabaseConnection.Params.Add('CHARSET=ansi');
           UserDatabaseConnection.UserName := 'adimn';
           UserDatabaseConnection.Password := Settings.CoreDBPassword;
        end;
     except
           result := false;
     end;

     {Create user data transaction component.}
     UserTransaction := TSQLTransaction.Create(parent);
     try
        {Link transaction to connection components.}
        UserDatabaseConnection.Transaction := UserTransaction;
        UserTransaction.DataBase := UserDatabaseConnection;
        UserTransaction.Action:= caCommit;
        UserTransaction.Active := false;
     except
           result := false;
     end;

     {Create user query component.}
     UserQuery := TSQLQuery.Create(parent);
     try
        UserQuery.DataBase := UserDatabaseConnection;
        UserQuery.Transaction := UserTransaction;
        UserQuery.UsePrimaryKeyAsKey := false;
        UserQuery.PacketRecords := -1;
     except
           result := false;
     end;

     {Atempt database connection.}
     try
        UserDatabaseConnection.Connected := true;
        UserDatabaseConnection.KeepConnection := true;
        result := UserDatabaseConnection.Connected;
     except
           {Regitster database error message.}
           Settings.DBErrorList.Add('Cannot connect to user database.');
           Settings.DBStatus := false;
           MainForm.UpdateSettingsList;
           {Send message to open main form.}
           Windows.SendMessageTimeout(HWND_BROADCAST, CoreMessage, 0, 0, SMTO_NORMAL or SMTO_ABORTIFHUNG, 500, Dummy);
           result := false;
     end;
end;

{Closed user database connection and free components.}
procedure CloseUserConnection;
begin
     {Close active connection components.}
     if Assigned(UserQuery) then begin
        UserQuery.Close;
        UserQuery.Free;
     end;
     if Assigned(UserTransaction) then begin
        UserTransaction.Active := false;
        UserTransaction.Free;
     end;
     if Assigned(UserDatabaseConnection) then begin
        UserDatabaseConnection.Connected := false;;
        UserDatabaseConnection.Free;
     end;
end;

{Create tools database connection.}
function CreateToolsDatabaseConnection(parent: TForm): boolean;
begin
     result := false;
     {Create user database connection component.}
     ToolsDatabaseConnection := TODBCConnection  .Create(parent);
     try
        if Settings.SQLServerIndex = 0 then begin;
           ToolsDatabaseConnection.Driver := 'Microsoft Access Driver (*.mdb)';
           ToolsDatabaseConnection.Params.Add('DBQ=' + Settings.DatabaseDrive +
                                           Settings.DatabaseDir +
                                           Settings.ToolsDB);
           ToolsDatabaseConnection.Params.Add('ExtendedAnsiSQL=1');
           ToolsDatabaseConnection.Params.Add('CHARSET=ansi');
           ToolsDatabaseConnection.UserName := 'adimn';
           ToolsDatabaseConnection.Password := Settings.ToolsDBPassword;
        end;
     except
           result := false;
     end;

     {Create user data transaction component.}
     ToolsTransaction := TSQLTransaction.Create(parent);
     try
        {Link transaction to connection components.}
        ToolsDatabaseConnection.Transaction := ToolsTransaction;
        ToolsTransaction.DataBase := ToolsDatabaseConnection;
        ToolsTransaction.Action:= caCommit;
        ToolsTransaction.Active := false;
     except
           result := false;
     end;

     {Create user query component.}
     ToolsQuery := TSQLQuery.Create(parent);
     try
        ToolsQuery.DataBase := ToolsDatabaseConnection;
        ToolsQuery.Transaction := ToolsTransaction;
        ToolsQuery.UsePrimaryKeyAsKey := false;
        ToolsQuery.PacketRecords := -1;
     except
           result := false;
     end;

     {Atempt database connection.}
     try
        ToolsDatabaseConnection.Connected := true;
        ToolsDatabaseConnection.KeepConnection := true;
        result := ToolsDatabaseConnection.Connected;
     except
           {Regitster database error message.}
           Settings.DBErrorList.Add('Cannot connect to tools database.');
           Settings.DBStatus := false;
           MainForm.UpdateSettingsList;
           {Send message to open main form.}
           Windows.SendMessageTimeout(HWND_BROADCAST, CoreMessage, 0, 0, SMTO_NORMAL or SMTO_ABORTIFHUNG, 500, Dummy);
           result := false;
     end;
end;

{Close and destroy tools database connection objects.}
procedure CloseToolsConnection;
begin
     {Close active connection components.}
     if Assigned(ToolsQuery) then begin
        ToolsQuery.Close;
        ToolsQuery.Free;
     end;
     if Assigned(ToolsTransaction) then begin
        ToolsTransaction.Active := false;
        ToolsTransaction.Free;
     end;
     if Assigned(ToolsDatabaseConnection) then begin
        ToolsDatabaseConnection.Connected := false;;
        ToolsDatabaseConnection.Free;
     end;
end;


{Create workbench database connection.}
function CreateWorkbenchDatabaseConnection(parent: TForm): boolean;
begin
     result := false;
     {Create user database connection component.}
     WorkbenchDatabaseConnection := TODBCConnection  .Create(parent);
     try
        if Settings.SQLServerIndex = 0 then begin
           WorkbenchDatabaseConnection.Driver := 'Microsoft Access Driver (*.mdb)';
           WorkbenchDatabaseConnection.Params.Add('DBQ=' + Settings.DatabaseDrive +
                                           Settings.DatabaseDir +
                                           Settings.BenchcheckDB);
           WorkbenchDatabaseConnection.Params.Add('ExtendedAnsiSQL=1');
           WorkbenchDatabaseConnection.Params.Add('CHARSET=ansi');
           WorkbenchDatabaseConnection.UserName := 'adimn';
           WorkbenchDatabaseConnection.Password := Settings.BenchcheckDBPassword;
        end;
     except
           result := false;
     end;

     {Create user data transaction component.}
     WorkbenchTransaction := TSQLTransaction.Create(parent);
     try
        {Link transaction to connection components.}
        WorkbenchDatabaseConnection.Transaction := WorkbenchTransaction;
        WorkbenchTransaction.DataBase := WorkbenchDatabaseConnection;
        WorkbenchTransaction.Action:= caCommit;
        WorkbenchTransaction.Active := false;
     except
           result := false;
     end;

     {Create user query component.}
     WorkbenchQuery := TSQLQuery.Create(parent);
     try
        WorkbenchQuery.DataBase := WorkbenchDatabaseConnection;
        WorkbenchQuery.Transaction := WorkbenchTransaction;
        WorkbenchQuery.UsePrimaryKeyAsKey := false;
        WorkbenchQuery.PacketRecords := -1;
     except
           result := false;
     end;

     {Atempt database connection.}
     try
        WorkbenchDatabaseConnection.Connected := true;
        WorkbenchDatabaseConnection.KeepConnection := true;
        result := WorkbenchDatabaseConnection.Connected;
     except
           {Regitster database error message.}
           Settings.DBErrorList.Add('Cannot connect to workbench database.');
           Settings.DBStatus := false;
           MainForm.UpdateSettingsList;
           {Send message to open main form.}
           Windows.SendMessageTimeout(HWND_BROADCAST, CoreMessage, 0, 0, SMTO_NORMAL or SMTO_ABORTIFHUNG, 500, Dummy);
           result := false;
     end;
end;

{Close and destroy workbench database connection objects.}
procedure CloseWorkbenchConnection;
begin
     {Close active connection components.}
     if Assigned(WorkbenchQuery) then begin
        WorkbenchQuery.Close;
        WorkbenchQuery.Free;
     end;
     if Assigned(WorkbenchTransaction) then begin
        WorkbenchTransaction.Active := false;
        WorkbenchTransaction.Free;
     end;
     if Assigned(WorkbenchDatabaseConnection) then begin
        WorkbenchDatabaseConnection.Connected := false;;
        WorkbenchDatabaseConnection.Free;
     end;
end;

end.

