unit IPservices;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFnDEF FPC}
  Windows,
{$ELSE}
  LCLIntf, LCLType, LMessages,
{$ENDIF}
  Controls, Graphics, SysUtils, Variants, Classes, Messages, Dialogs, winsock,
  windows;

  {This code requires at least Win98/ME/2K, 95 OSR 2 or NT service pack #3
  as WinSock 2 is used (WS2_32.DLL) }


// Constants found in manual on non-officially documented M$ Winsock functions
Const SIO_GET_INTERFACE_LIST = $4004747F;
      IFF_UP                 = $00000001;
      IFF_BROADCAST          = $00000002;
      IFF_LOOPBACK           = $00000004;
      IFF_POINTTOPOINT       = $00000008;
      IFF_MULTICAST          = $00000010;


Type SockAddr_Gen          = Packed Record
                               AddressIn             : SockAddr_In;
                               Padding               : Packed Array [0..7] of Byte;
                             end;

     Interface_Info        = Record
                               iiFlags               : u_Long;
                               iiAddress             : SockAddr_Gen;
                               iiBroadcastAddress    : SockAddr_Gen;
                               iiNetmask             : SockAddr_Gen;
                             end;

     TNetworkInterface     = record
                                ComputerName          : String;
                                AddrIP                : String;
                                SubnetMask            : String;
                                AddrNet               : String;
                                AddrLimitedBroadcast  : String;
                                AddrDirectedBroadcast : String;
                                IsInterfaceUp         : Boolean;
                                BroadcastSupport      : Boolean;
                                IsLoopback            : Boolean;
                                UseInterface          : Boolean;
                             end;

     TNetworkInterfaceList = Array of TNetworkInterface;


type
  TSunB = packed record
    s_b1, s_b2, s_b3, s_b4: byte;
  end;

  TSunW = packed record
    s_w1, s_w2: word;
  end;

  PIPAddr = ^TIPAddr;
  TIPAddr = record
    case integer of
      0: (S_un_b: TSunB);
      1: (S_un_w: TSunW);
      2: (S_addr: longword);
  end;

IPAddr = TIPAddr;

function WSAIoctl (aSocket              : TSocket;
                   aCommand             : DWord;
                   lpInBuffer           : Pointer;
                   dwInBufferLen        : DWord;
                   lpOutBuffer          : Pointer;
                   dwOutBufferLen       : DWord;
                   lpdwOutBytesReturned : LPDWord;
                   lpOverLapped         : Pointer;
                   lpOverLappedRoutine  : Pointer) : Integer; stdcall; external 'WS2_32.DLL';
function IcmpCreateFile : THandle; stdcall; external 'icmp.dll';
function IcmpCloseHandle (icmpHandle : THandle) : boolean;
            stdcall; external 'icmp.dll';
function IcmpSendEcho(IcmpHandle : THandle; DestinationAddress : IPAddr;
                      RequestData : Pointer; RequestSize : Smallint;
                      RequestOptions : pointer;
                      ReplyBuffer : Pointer;
                      ReplySize : DWORD;
                      Timeout : DWORD) : DWORD; stdcall; external 'icmp.dll';
function Ping(InetAddress : string) : boolean;
function Fetch(var AInput: string;
                      const ADelim: string = ' ';
                      const ADelete: Boolean = true) : string;
function GetIPFromHost(var HostName, IPaddr, WSAErr: string): Boolean;
function HostToIPstr(Name: string; var Ip: string): Boolean;
function IsValidIP(InputIP: string; var OutputIP: string): Boolean;
function GetNetworkInterfaces (Var aNetworkInterfaceList : tNetworkInterfaceList): Boolean;
function PortIsOpen(dwPort : Word; ipAddressStr:AnsiString) : boolean;

procedure StringToTInAddr(AIP: string; var AInAddr);


var
  err : string;

implementation



// Recover host name & local IP address.
function GetIPFromHost(var HostName, IPaddr, WSAErr: string): Boolean;
var
  Hname: array[0..100] of AnsiChar;
  HEnt: pHostEnt;
  WSAData: TWSAData;
  i: integer;
begin
  result := false;
  if WSAStartup($0101, WSAData) <> 0 then begin
    WSAErr := 'Winsock is not responding.';
    Exit;
  end;
  IPaddr := '';
  if gethostname(Hname, sizeof(Hname)) = 0 then
  begin
      HostName := strpas(Hname);
      HEnt := GetHostByName(Hname);
      for i := 0 to HEnt^.h_length - 1 do
        IPaddr :=
          Concat(IPaddr, inttostr(Ord(HEnt^.h_addr_list^[i])) + '.');
          SetLength(IPaddr, Length(IPaddr) - 1);
          Result := true;
  end
  else begin
       case WSAGetLastError of
          WSANOTINITIALISED: WSAErr := 'WSANotInitialised';
          WSAENETDOWN     : WSAErr := 'WSAENetDown';
          WSAEINPROGRESS  : WSAErr := 'WSAEInProgress';
       end;
  end;
  WSACleanup;
end;



// Converts hostname to IP address string.
function HostToIPstr(Name: string; var Ip: string): Boolean;
var
  wsdata : TWSAData;
  hostName : array [0..255] of AnsiChar;
  hostEnt : PHostEnt;
  addr : PAnsiChar;
begin
  WSAStartup ($0101, wsdata);
  try
    gethostname (hostName, sizeof (hostName));
    StrPCopy(hostName, Name);
    hostEnt := gethostbyname (hostName);
    if Assigned (hostEnt) then
      if Assigned (hostEnt^.h_addr_list) then begin
        addr := hostEnt^.h_addr_list^;
        if Assigned (addr) then begin
          IP := Format ('%d.%d.%d.%d', [byte (addr [0]),
          byte (addr [1]), byte (addr [2]), byte (addr [3])]);
          Result := True;
        end
        else
          Result := False;
      end
      else
      begin
        Result := False;
      end
    else begin
      Result := False;
      raise EArgumentException.Create('Server Host ' + Name + ' Cannot Be Found.');
    end;
  finally
    WSACleanup;
  end
end;



// Determin if IP address entered by user is valid.
function IsValidIP(InputIP: string; var OutputIP: string): Boolean;
var
  error : boolean;
  pIP : PChar;
  start, stop, net1, net2, host1, host2: integer;
begin
  result := false; error := false;
  start := 1; stop := 1; net1 := 0; net2 := 0; host1 := 0; host2 := 0;
  pIP := Addr(InputIP[stop]);

  // Extract first net IP
  while (pIP[stop] <> '.') and (stop < length(InputIP)) do
  begin
    inc(stop);
  end;
  inc(stop);
  if (stop - start) > 3 then // Error if number more than 3 digits
    error := true
  else
    net1 := strtoint(trimright(copy(InputIP, start, (stop - start))));

  // Extarct second net IP
  if not error  then
  begin
    inc(stop);    // Skip dot
    start := stop;
    while (pIP[stop] <> '.') and (stop < length(InputIP)) do
    begin
      inc(stop);
    end;
  end;
  if not error then
  begin
    inc(stop);
    if (stop - start) > 3 then // Error if number more than 3 digits
      error := true
    else
      net2 := strtoint(trimright(copy(InputIP, start, (stop - start))));
  end;

  // Extarct first host IP
  if not error  then
  begin
    inc(stop);    // Skip dot
    start := stop;
    while (pIP[stop] <> '.') and (stop < length(InputIP)) do
    begin
      inc(stop);
    end;
  end;

  if not error then
  begin
    inc(stop);
    if (stop - start) > 3 then // Error if number more than 3 digits
      error := true
    else
      host1 := strtoint(trimright(copy(InputIP, start, (stop - start))));
  end;

  // Extract second host IP
  if not error then
  begin
    inc(stop);    // Skip dot
    start := stop;
    while stop < length(InputIP) do
    begin
      inc(stop);
    end;
  end;

  if not error then
  begin
    inc(stop);
    if (stop - start) > 3 then // Error if number more than 3 digits
      error := true
    else
      host2 := strtoint(trimright(copy(InputIP, start, (stop - start))));
  end;

  // Verify IP's are between 0 & 255 and return complete Ip address
  if not error then
  begin
  if (net1 < 0) or (net1 > 255) or (net2 < 0) or (net2 > 255) or
      (host1 < 0) or (host1 > 255) or (host2 < 0) or (host2 > 255) then
      begin
        error := true;
      end
      else
      begin
        OutputIP := inttostr(net1) + '.' + inttostr(net2) + '.' +
          inttostr(host1) + '.' + inttostr(host2);
        result := true;
      end;
  end;

  if error then
  begin
    raise EArgumentException.Create('Not A Valid IP Address.');
    OutputIP := '';
    exit
  end;
end;



// Fetch
function Fetch(var AInput: string;
                      const ADelim: string = ' ';
                      const ADelete: Boolean = true)
 : string;
 //
var
  iPos: Integer;
begin
  if ADelim = #0 then begin
    // AnsiPos does not work with #0
    iPos := Pos(ADelim, AInput);
  end else begin
    iPos := Pos(ADelim, AInput);
  end;
  if iPos = 0 then begin
    Result := AInput;
    if ADelete then begin
      AInput := '';
    end;
  end else begin
    result := Copy(AInput, 1, iPos - 1);
    if ADelete then begin
      Delete(AInput, 1, iPos + Length(ADelim) - 1);
    end;
  end;
end;



// Converts IP address string to real address number.
procedure StringToTInAddr(AIP: string; var AInAddr);
var
  straddr : array [0..255] of AnsiChar;
  phe: PHostEnt;
  pac: PAnsiChar;
  GInitData: TWSAData;
begin
  WSAStartup($101, GInitData);
  try
    gethostname (straddr, sizeof (straddr));
    StrPCopy(straddr, AIP);
    phe := GetHostByName(straddr);
    if Assigned(phe) then
    begin
      pac := phe^.h_addr_list^;
      if Assigned(pac) then
      begin
        with TIPAddr(AInAddr).S_un_b do begin
          s_b1 := Byte(pac[0]);
          s_b2 := Byte(pac[1]);
          s_b3 := Byte(pac[2]);
          s_b4 := Byte(pac[3]);
        end;
      end
      else
      begin
        raise Exception.Create('Error getting IP from HostName');
      end;
    end
    else
    begin
      raise Exception.Create('Error getting HostName');
    end;
  except
    FillChar(AInAddr, SizeOf(AInAddr), #0);
  end;
  WSACleanup;
end;



// Pings specified IP address.
function Ping(InetAddress : string) : boolean;
var
 Handle : THandle;
 InAddr : IPAddr;
 DW : DWORD;
 rep : array[1..128] of byte;
begin
  result := false;
  Handle := IcmpCreateFile;
  if Handle = INVALID_HANDLE_VALUE then
   Exit;
  StringToTInAddr(InetAddress, InAddr);
  DW := IcmpSendEcho(Handle, InAddr, nil, 0, nil, @rep, 128, 0);
  Result := (DW <> 0);
  IcmpCloseHandle(Handle);
end;



// Returns a complete list the of available network interfaces on a system (IPv4)
// Copyright by Dr. Jan Schulz, 23-26th March 2007
// This version can be used for free and non-profit projects. In any other case get in contact
// Written with information retrieved from MSDN
// www.code10.net
Function GetNetworkInterfaces (Var aNetworkInterfaceList : tNetworkInterfaceList): Boolean;
Var aSocket             : TSocket;
    aWSADataRecord      : WSAData;
    NoOfInterfaces      : Integer;
    NoOfBytesReturned   : u_Long;
    InterfaceFlags      : u_Long;
    NameLength          : DWord;
    pAddrIP             : SockAddr_In;
    pAddrSubnetMask     : SockAddr_In;
    pAddrBroadcast      : Sockaddr_In;
    DirBroadcastDummy   : In_Addr;
    NetAddrDummy        : In_Addr;
    Buffer              : Array [0..30] of Interface_Info;
    i                   : Integer;
Begin
  Result := False;
  SetLength (aNetworkInterfaceList, 0);

  // Startup of old the WinSock
  // WSAStartup ($0101, aWSADataRecord);

  // Startup of WinSock2
  WSAStartup(MAKEWORD(2, 0), aWSADataRecord);

  // Open a socket
  aSocket := Socket (AF_INET, SOCK_STREAM, 0);

  // If impossible to open a socket, not worthy to go any further
  If (aSocket = INVALID_SOCKET) THen Exit;

  Try
    If WSAIoCtl (aSocket, SIO_GET_INTERFACE_LIST, NIL, 0,
                 @Buffer, 1024, @NoOfBytesReturned, NIL, NIL) <> SOCKET_ERROR THen
    Begin
      NoOfInterfaces := NoOfBytesReturned  Div SizeOf (Interface_Info);
      SetLength (aNetworkInterfaceList, NoOfInterfaces);

      // For each of the identified interfaces get:
      For i := 0 to NoOfInterfaces - 1 do
      Begin

        With aNetworkInterfaceList[i] do
        Begin

          // Get the name of the machine
          NameLength := MAX_COMPUTERNAME_LENGTH + 1;
          SetLength (ComputerName, NameLength)  ;
          If Not GetComputerName (PChar (Computername), NameLength) THen ComputerName := '';

          // Get the IP address
          pAddrIP                  := Buffer[i].iiAddress.AddressIn;
          AddrIP                   := string(inet_ntoa (pAddrIP.Sin_Addr));

          // Get the subnet mask
          pAddrSubnetMask          := Buffer[i].iiNetMask.AddressIn;
          SubnetMask               := string(inet_ntoa (pAddrSubnetMask.Sin_Addr));

          // Get the limited broadcast address
          pAddrBroadcast           := Buffer[i].iiBroadCastAddress.AddressIn;
          AddrLimitedBroadcast     := string(inet_ntoa (pAddrBroadcast.Sin_Addr));

          // Calculate the net and the directed broadcast address
          NetAddrDummy.S_addr      := Buffer[i].iiAddress.AddressIn.Sin_Addr.S_Addr;
          NetAddrDummy.S_addr      := NetAddrDummy.S_addr And Buffer[i].iiNetMask.AddressIn.Sin_Addr.S_Addr;
          DirBroadcastDummy.S_addr := NetAddrDummy.S_addr Or Not Buffer[i].iiNetMask.AddressIn.Sin_Addr.S_Addr;

          AddrNet                  := string(inet_ntoa ((NetAddrDummy)));
          AddrDirectedBroadcast    := string(inet_ntoa ((DirBroadcastDummy)));

          // From the evaluation of the Flags we receive more information
          InterfaceFlags           := Buffer[i].iiFlags;

          // Is the network interface up or down ?
          If (InterfaceFlags And IFF_UP) = IFF_UP THen IsInterfaceUp := True
                                                  Else IsInterfaceUp := False;

          // Does the network interface support limited broadcasts ?
          If (InterfaceFlags And IFF_BROADCAST) = IFF_BROADCAST THen BroadcastSupport := True
                                                                Else BroadcastSupport := False;

          // Is the network interface a loopback interface ?
          If (InterfaceFlags And IFF_LOOPBACK) = IFF_LOOPBACK THen IsLoopback := True
                                                              Else IsLoopback := False;
          UseInterface := true;
        end;
      end;
    end;
  Except
    //Result := False;
  end;

  // Cleanup the mess
  CloseSocket (aSocket);
  WSACleanUp;
  Result := True;
end;



// Returns true if port is open at ip address
function PortIsOpen(dwPort : Word; ipAddressStr:AnsiString) : boolean;
var
  client : sockaddr_in;
  sock   : Integer;

  ret    : Integer;
  wsdata : WSAData;
begin
 Result:=False;
 ret := WSAStartup($0002, wsdata); //initiates use of the Winsock DLL
  if ret<>0 then exit;
  try
    client.sin_family      := AF_INET;  //Set the protocol to use , in this case (IPv4)
    client.sin_port        := htons(dwPort); //convert to TCP/IP network byte order (big-endian)
    client.sin_addr.s_addr := inet_addr(PAnsiChar(ipAddressStr));  //convert to IN_ADDR  structure
    sock  :=socket(AF_INET, SOCK_STREAM, 0);    //creates a socket
    Result:=connect(sock,client,SizeOf(client))=0;  //establishes a connection to a specified socket
  finally
  WSACleanup;
  end;
end;

end.
